@@ -1,11 +1,12 @@
-/**
-* Copyright 2005-2018 ECMWF
-*
-* Licensed under the GNU Lesser General Public License which
-* incorporates the terms and conditions of version 3 of the GNU
-* General Public License.
-* See LICENSE and gpl-3.0.txt for details.
-*/
+/*
+ * Copyright 2005-2019 ECMWF.
+ *
+ * This software is licensed under the terms of the Apache Licence Version 2.0
+ * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ *
+ * In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
+ * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
+ */
 
 /*****************************************
  *  Enrico Fucile
@@ -34,6 +35,7 @@
    MEMBERS    = grib_vdarray* numericValues
    MEMBERS    = grib_vsarray* stringValues
    MEMBERS    = grib_viarray* elementsDescriptorsIndex
+   MEMBERS    = char* cname
 
    END_CLASS_DEF
 
@@ -79,6 +81,7 @@ typedef struct grib_accessor_bufr_data_element {
 	grib_vdarray* numericValues;
 	grib_vsarray* stringValues;
 	grib_viarray* elementsDescriptorsIndex;
+	char* cname;
 } grib_accessor_bufr_data_element;
 
 extern grib_accessor_class* grib_accessor_class_gen;
@@ -161,6 +164,7 @@ static grib_accessor* make_clone(grib_accessor* a,grib_section* s,int* err)
     grib_accessor* attribute=NULL;
     grib_accessor_bufr_data_element* elementAccessor;
     grib_accessor_bufr_data_element* self;
+    char* copied_name = NULL;
     int i;
     grib_action creator = {0, };
     creator.op         = "bufr_data_element";
@@ -173,7 +177,8 @@ static grib_accessor* make_clone(grib_accessor* a,grib_section* s,int* err)
     *err=0;
 
     the_clone = grib_accessor_factory(s, &creator, 0, NULL);
-    the_clone->name=grib_context_strdup(a->context,a->name);
+    copied_name = grib_context_strdup(a->context,a->name);
+    the_clone->name=copied_name;
     elementAccessor=(grib_accessor_bufr_data_element*)the_clone;
     self=(grib_accessor_bufr_data_element*)a;
     the_clone->flags=a->flags;
@@ -188,6 +193,7 @@ static grib_accessor* make_clone(grib_accessor* a,grib_section* s,int* err)
     elementAccessor->numericValues=self->numericValues;
     elementAccessor->stringValues=self->stringValues;
     elementAccessor->elementsDescriptorsIndex=self->elementsDescriptorsIndex;
+    elementAccessor->cname = copied_name; /* ECC-765 */
 
     i=0;
     while (a->attributes[i]) {
@@ -256,10 +262,11 @@ void accessor_bufr_data_element_set_elementsDescriptorsIndex(grib_accessor* a,gr
 
 static void init(grib_accessor* a, const long len, grib_arguments* params)
 {
-
+    grib_accessor_bufr_data_element* self = (grib_accessor_bufr_data_element*)a;
     a->length = 0;
     a->flags |= GRIB_ACCESSOR_FLAG_BUFR_DATA;
     /* a->flags |= GRIB_ACCESSOR_FLAG_READ_ONLY; */
+    self->cname = NULL;
 }
 
 static void dump(grib_accessor* a, grib_dumper* dumper)
@@ -318,7 +325,7 @@ static int pack_string_array(grib_accessor*a , const char**  v, size_t *len)
     if (self->compressedData) {
         idx=((int)self->numericValues->v[self->index]->v[0]/1000-1)/self->numberOfSubsets;
         if (*len!=1 && *len!=self->numberOfSubsets) {
-            grib_context_log(c,GRIB_LOG_ERROR,"Number of values mismatch for %s: are %ld should be %ld",
+            grib_context_log(c,GRIB_LOG_ERROR,"Number of values mismatch for '%s': %ld strings provided but expected %ld (=number of subsets)",
                     self->descriptors->v[self->elementsDescriptorsIndex->v[0]->v[idx]]->shortName,*len,self->numberOfSubsets);
             return GRIB_ARRAY_TOO_SMALL;
         }
@@ -481,7 +488,7 @@ static int pack_double(grib_accessor* a, const double* val, size_t *len)
     if (self->compressedData) {
         count=*len;
         if (count!=1 && count!=self->numberOfSubsets) {
-            grib_context_log(c,GRIB_LOG_ERROR,"Number of values mismatch for %s: are %ld should be %ld",
+            grib_context_log(c,GRIB_LOG_ERROR,"Number of values mismatch for '%s': %ld doubles provided but expected %ld (=number of subsets)",
                     self->descriptors->v[self->elementsDescriptorsIndex->v[0]->v[self->index]]->shortName,count,self->numberOfSubsets);
             return GRIB_ARRAY_TOO_SMALL;
         }
@@ -510,7 +517,7 @@ static int pack_long(grib_accessor* a, const long* val, size_t *len)
     if (self->compressedData) {
         count=*len;
         if (count!=1 && count!=self->numberOfSubsets) {
-            grib_context_log(c,GRIB_LOG_ERROR,"Number of values mismatch for %s: are %ld should be %ld",
+            grib_context_log(c,GRIB_LOG_ERROR,"Number of values mismatch for '%s': %ld integers provided but expected %ld (=number of subsets)",
                     self->descriptors->v[self->elementsDescriptorsIndex->v[0]->v[self->index]]->shortName,count,self->numberOfSubsets);
             return GRIB_ARRAY_TOO_SMALL;
         }
@@ -602,10 +609,12 @@ static int  get_native_type(grib_accessor* a)
 
 static void destroy(grib_context* ct, grib_accessor* a)
 {
+    grib_accessor_bufr_data_element* self = (grib_accessor_bufr_data_element*)a;
     int i=0;
+    if (self->cname) grib_context_free(ct,self->cname); /* ECC-765 */
     while (i<MAX_ACCESSOR_ATTRIBUTES && a->attributes[i]) {
-        grib_context_log(ct,GRIB_LOG_DEBUG,"deleting attribute %s->%s",a->name,a->attributes[i]->name);
-        /* printf("+++++ %s\n",a->attributes[i]->name); */
+        /*grib_context_log(ct,GRIB_LOG_DEBUG,"deleting attribute %s->%s",a->name,a->attributes[i]->name);*/
+        /*printf("bufr_data_element destroy %s %p\n", a->attributes[i]->name, (void*)a->attributes[i]);*/
         grib_accessor_delete(ct,a->attributes[i]);
         a->attributes[i]=NULL;
         i++;
