@@ -1,5 +1,5 @@
 /*
- * Copyright 2005-2018 ECMWF.
+ * Copyright 2005-2019 ECMWF.
  *
  * This software is licensed under the terms of the Apache Licence Version 2.0
  * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
@@ -208,8 +208,8 @@ static grib_trie* load_bufr_elements_table(grib_accessor* a, int* err)
     GRIB_MUTEX_LOCK(&mutex1);
 
     if (*masterDir!=0) {
-        char name[1024]={0,};
-        char recomposed[1024]={0,};
+        char name[4096]={0,};
+        char recomposed[4096]={0,};
         sprintf(name,"%s/%s",masterDir,self->dictionary);
         grib_recompose_name(h, NULL,name, recomposed,0);
         filename=grib_context_full_defs_path(c,recomposed);
@@ -219,7 +219,7 @@ static grib_trie* load_bufr_elements_table(grib_accessor* a, int* err)
 
     if (*localDir!=0) {
         char localRecomposed[1024]={0,};
-        char localName[1024]={0,};
+        char localName[2048]={0,};
         sprintf(localName,"%s/%s",localDir,self->dictionary);
         grib_recompose_name(h, NULL,localName, localRecomposed,0);
         localFilename=grib_context_full_defs_path(c,localRecomposed);
@@ -239,7 +239,7 @@ static grib_trie* load_bufr_elements_table(grib_accessor* a, int* err)
 
     dictionary=(grib_trie*)grib_trie_get(c->lists,dictName);
     if (dictionary) {
-        grib_context_log(c,GRIB_LOG_DEBUG,"using dictionary %s from cache",self->dictionary);
+        /*grib_context_log(c,GRIB_LOG_DEBUG,"using dictionary %s from cache",self->dictionary);*/
         goto the_end;
     } else {
         grib_context_log(c,GRIB_LOG_DEBUG,"using dictionary %s from file %s",self->dictionary,filename);
@@ -275,7 +275,7 @@ the_end:
     return dictionary;
 }
 
-static int convert_type(char* stype)
+static int convert_type(const char* stype)
 {
     int ret=BUFR_DESCRIPTOR_TYPE_UNKNOWN;
     switch (stype[0]) {
@@ -301,40 +301,37 @@ static int convert_type(char* stype)
     return ret;
 }
 
+static long atol_fast(const char *input)
+{
+    if(strcmp(input,"0")==0)
+        return 0;
+    return atol(input);
+}
 static int bufr_get_from_table(grib_accessor* a,bufr_descriptor* v)
 {
     int ret=0;
     char** list=0;
     char code[7]={0};
-    grib_context* c;
-    char* str=NULL;
 
-    grib_trie* table;
-
-    table=load_bufr_elements_table(a,&ret);
+    grib_trie* table = load_bufr_elements_table(a,&ret);
     if (ret) return ret;
 
-    c=a->context;
-
     sprintf(code,"%06ld",v->code);
 
     list=(char**)grib_trie_get(table,code);
     if (!list) return GRIB_NOT_FOUND;
 
-    v->shortName=grib_context_strdup(c,list[1]);
+    v->shortName=grib_context_strdup(a->context,list[1]);
     v->type=convert_type(list[2]);
     /* v->name=grib_context_strdup(c,list[3]);  See ECC-489 */
-    v->units=grib_context_strdup(c,list[4]);
-    str=grib_context_strdup(c,list[5]);
-    v->scale=atol(str);
-    grib_context_free(c,str);
+    v->units=grib_context_strdup(a->context,list[4]);
+
+    /* ECC-985: Scale and reference are often 0 so we can reduce calls to atol */
+    v->scale=atol_fast(list[5]);
     v->factor=grib_power(-v->scale,10);
-    str=grib_context_strdup(c,list[6]);
-    v->reference=atol(str);
-    grib_context_free(c,str);
-    str=grib_context_strdup(c,list[7]);
-    v->width=atol(str);
-    grib_context_free(c,str);
+
+    v->reference=atol_fast(list[6]);
+    v->width=atol(list[7]);
 
     return ret;
 }
@@ -361,7 +358,7 @@ bufr_descriptor* accessor_bufr_elements_table_get_descriptor(grib_accessor* a,in
     if (!a) return NULL;
 
     c=a->context;
-
+    DebugAssert(c);
     v=(bufr_descriptor*)grib_context_malloc_clear(c,sizeof(bufr_descriptor));
     if (!v) {
         grib_context_log(c,GRIB_LOG_ERROR,
