@@ -1,5 +1,5 @@
 /*
- * Copyright 2005-2018 ECMWF.
+ * Copyright 2005-2019 ECMWF.
  *
  * This software is licensed under the terms of the Apache Licence Version 2.0
  * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
@@ -294,7 +294,8 @@ static int unpack_double(grib_accessor* a, double* val, size_t *len)
             != GRIB_SUCCESS)
         return ret;
 
-    if((ret = grib_get_long_internal(gh,self->ieee_floats,&ieee_floats)) != GRIB_SUCCESS)
+    /* ECC-774: don't use grib_get_long_internal */
+    if((ret = grib_get_long(gh,self->ieee_floats,&ieee_floats)) != GRIB_SUCCESS)
         return ret;
 
     if((ret = grib_get_double_internal(gh,self->laplacianOperator,&laplacianOperator))
@@ -423,6 +424,10 @@ static int unpack_double(grib_accessor* a, double* val, size_t *len)
                     bits_per_value)*s)+reference_value)*scals[lup];
             val[i++] =  d * (double) ((grib_decode_unsigned_long(lres, &lpos,
                     bits_per_value)*s)+reference_value)*scals[lup];
+            /* These values should always be zero, but as they are packed,
+               it is necessary to force them back to zero */
+            if (mmax == 0)
+                val[i-1] = 0;
             lup++;
         }
 #endif
