@@ -1,5 +1,5 @@
 /*
-* Copyright 2005-2018 ECMWF.
+* Copyright 2005-2019 ECMWF.
 *
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
@@ -234,6 +234,7 @@ static size_t __expand(grib_accessor* a, bufr_descriptors_array* unexpanded, buf
     if (BUFR_DESCRIPTORS_ARRAY_USED_SIZE(unexpanded)==0) return 0;
 
     us=grib_bufr_descriptor_clone(grib_bufr_descriptors_array_get(unexpanded,0));
+    us->context = c;
 
     *err=0;
 #if MYDEBUG
@@ -264,6 +265,7 @@ static size_t __expand(grib_accessor* a, bufr_descriptors_array* unexpanded, buf
         }
         grib_context_free(c,v);
         inner_expanded=do_expand(a,inner_unexpanded,ccp,err);
+        if (*err) return 0;
         grib_bufr_descriptors_array_delete(inner_unexpanded);
 #if MYDEBUG
         for (i=0;i<inner_expanded->n;i++) {
@@ -291,6 +293,14 @@ static size_t __expand(grib_accessor* a, bufr_descriptors_array* unexpanded, buf
             size=0;
             inner_unexpanded=grib_bufr_descriptors_array_new(c,DESC_SIZE_INIT,DESC_SIZE_INCR);
 
+            /* Number of descriptors to replicate cannot be more than what's left */
+            if (us->X+1 > unexpanded->n) {
+                grib_context_log(c, GRIB_LOG_ERROR,
+                                 "Delayed replication: %06ld: expected %d but only found %lu elements",
+                                 u->code, us->X, unexpanded->n - 1);
+                *err = GRIB_DECODING_ERROR;
+                return 0;
+            }
             for (j=0;j<us->X+1;j++) {
                 DESCRIPTORS_POP_FRONT_OR_RETURN(unexpanded, u0);
                 grib_bufr_descriptors_array_push(inner_unexpanded,u0);
@@ -300,6 +310,7 @@ static size_t __expand(grib_accessor* a, bufr_descriptors_array* unexpanded, buf
 #endif
             }
             inner_expanded=do_expand(a,inner_unexpanded,ccp,err);
+            if (*err) return 0;
             grib_bufr_descriptors_array_delete(inner_unexpanded);
             size=BUFR_DESCRIPTORS_ARRAY_USED_SIZE(inner_expanded);
 #if MYDEBUG
@@ -320,7 +331,7 @@ static size_t __expand(grib_accessor* a, bufr_descriptors_array* unexpanded, buf
 #endif
             grib_bufr_descriptor_delete(u);
             size=us->X*us->Y;
-            ur=(bufr_descriptor**)grib_context_malloc_clear(c,us->X*sizeof(bufr_descriptor));
+            ur=(bufr_descriptor**)grib_context_malloc_clear(c,us->X*sizeof(bufr_descriptor*));
             for (j=0;j<us->X;j++) {
                 DESCRIPTORS_POP_FRONT_OR_RETURN(unexpanded,ur[j]);
 #if MYDEBUG
@@ -342,6 +353,7 @@ static size_t __expand(grib_accessor* a, bufr_descriptors_array* unexpanded, buf
             for (i=0;i<us->X;i++) grib_bufr_descriptor_delete(ur[i]);
             grib_context_free(c,ur);
             inner_expanded=do_expand(a,inner_unexpanded,ccp,err);
+            if (*err) return 0;
             grib_bufr_descriptors_array_delete(inner_unexpanded);
 #if MYDEBUG
             for (i=0;i<inner_expanded->n;i++) {
@@ -500,6 +512,7 @@ static bufr_descriptors_array* do_expand(grib_accessor* a,bufr_descriptors_array
 #endif
     while (unexpanded->n) {
         __expand(a,unexpanded,expanded,ccp,err);
+        if (*err) return NULL;
     }
 #if MYDEBUG
     {
@@ -623,6 +636,7 @@ static int expand(grib_accessor* a)
     ccp.associatedFieldWidth=0;
     ccp.newStringWidth=0;
     self->expanded=do_expand(a,unexpanded,&ccp,&err);
+    if (err) return err;
     grib_context_expanded_descriptors_list_push(c,key,self->expanded,unexpanded_copy);
     grib_bufr_descriptors_array_delete(unexpanded);
 
