@@ -1,5 +1,5 @@
 /*
- * Copyright 2005-2018 ECMWF.
+ * Copyright 2005-2019 ECMWF.
  *
  * This software is licensed under the terms of the Apache Licence Version 2.0
  * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
@@ -58,7 +58,6 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 
-/* Microsoft Windows Visual Studio support */
 #include "eccodes_windef.h"
 
 #ifndef ECCODES_ON_WINDOWS
@@ -105,7 +104,6 @@
 
 #include <limits.h>
 #include <errno.h>
-#include <stdarg.h>
 #include <stdlib.h>
 #include <ctype.h>
 
@@ -1054,6 +1052,7 @@ struct grib_context
     int                             ieee_packing;
     int                             bufrdc_mode;
     int                             bufr_set_to_missing_if_out_of_range;
+    int                             bufr_multi_element_constant_arrays;
     FILE*                           log_stream;
     grib_trie*                      classes;
     grib_trie*                      lists;
@@ -1446,7 +1445,7 @@ struct grib_codetable {
 };
 
 typedef struct grib_smart_table_entry {
-  int   code;
+  /*int   code;*/
   char* abbreviation;
   char* column[MAX_SMART_TABLE_COLUMNS];
 } grib_smart_table_entry;
