@@ -1,5 +1,5 @@
 /*
- * Copyright 2005-2018 ECMWF.
+ * Copyright 2005-2019 ECMWF.
  *
  * This software is licensed under the terms of the Apache Licence Version 2.0
  * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
@@ -257,11 +257,11 @@ grib_handle* grib_handle_new_from_samples( grib_context* c, const char* name )
     grib_context_set_handle_total_count(c,0);
 
     /*
-       g = grib_internal_template(c,name);
-       if(g) return g;
+     * g = grib_internal_sample(c,name);
+     * if(g) return g;
      */
     if (c->debug) {
-        printf("ECCODES DEBUG: grib_handle_new_from_samples '%s'\n", name);
+        fprintf(stderr, "ECCODES DEBUG: grib_handle_new_from_samples '%s'\n", name);
     }
 
     g = grib_external_template( c,name );
@@ -280,11 +280,11 @@ grib_handle* codes_bufr_handle_new_from_samples ( grib_context* c, const char* n
     grib_context_set_handle_total_count(c,0);
 
     /*
-       g = grib_internal_template(c,name);
-       if(g) return g;
+     *  g = grib_internal_sample(c,name);
+     *  if(g) return g;
      */
     if (c->debug) {
-        printf("ECCODES DEBUG: grib_handle_new_from_samples '%s'\n", name);
+        fprintf(stderr, "ECCODES DEBUG: grib_handle_new_from_samples '%s'\n", name);
     }
 
     g=bufr_external_template ( c,name );
@@ -369,15 +369,15 @@ static int determine_product_kind(grib_handle* h, ProductKind* prod_kind)
 {
     int err = 0;
     size_t len = 0;
-    char prod_kind_str[256]={0,};
-    err = grib_get_length(h, "kindOfProduct", &len);
+    err = grib_get_length(h, "identifier", &len);
     if (!err) {
-        err = grib_get_string(h, "kindOfProduct", prod_kind_str, &len);
-        if      (grib_inline_strcmp(prod_kind_str, "GRIB")==0)  *prod_kind = PRODUCT_GRIB;
-        else if (grib_inline_strcmp(prod_kind_str, "BUFR")==0)  *prod_kind = PRODUCT_BUFR;
-        else if (grib_inline_strcmp(prod_kind_str, "METAR")==0) *prod_kind = PRODUCT_METAR;
-        else if (grib_inline_strcmp(prod_kind_str, "GTS")==0)   *prod_kind = PRODUCT_GTS;
-        else if (grib_inline_strcmp(prod_kind_str, "TAF")==0)   *prod_kind = PRODUCT_TAF;
+        char id_str[64]={0,};
+        err = grib_get_string(h, "identifier", id_str, &len);
+        if      (grib_inline_strcmp(id_str, "GRIB")==0)  *prod_kind = PRODUCT_GRIB;
+        else if (grib_inline_strcmp(id_str, "BUFR")==0)  *prod_kind = PRODUCT_BUFR;
+        else if (grib_inline_strcmp(id_str, "METAR")==0) *prod_kind = PRODUCT_METAR;
+        else if (grib_inline_strcmp(id_str, "GTS")==0)   *prod_kind = PRODUCT_GTS;
+        else if (grib_inline_strcmp(id_str, "TAF")==0)   *prod_kind = PRODUCT_TAF;
         else *prod_kind = PRODUCT_ANY;
     }
     return err;
@@ -448,6 +448,15 @@ grib_handle* grib_handle_new_from_message ( grib_context* c, const void* data, s
     if (determine_product_kind(h, &product_kind) == GRIB_SUCCESS) {
         h->product_kind = product_kind;
     }
+
+    if (h->product_kind == PRODUCT_GRIB) {
+        if (!grib_is_defined(h,"7777")) {
+            grib_context_log (c,GRIB_LOG_ERROR,"grib_handle_new_from_message: No final 7777 in message!");
+            /* TODO: Return NULL. An incomplete message is no use to anyone.
+             * But first check the MARS Client and other applications
+             */
+        }
+    }
     return h;
 }
 
@@ -782,7 +791,9 @@ static grib_handle* grib_handle_new_from_file_multi ( grib_context* c, FILE* f,i
         gl->gts_header_len=gtslen;
         grib_context_free ( c,save_gts_header );
         gtslen=0;
-    } else gl->gts_header=NULL;
+    } else {
+        gl->gts_header=NULL;
+    }
 
     return gl;
 }
@@ -934,11 +945,16 @@ grib_handle* bufr_new_from_file( grib_context* c, FILE* f,int *error )
     void *data = NULL;
     size_t olen = 0;
     grib_handle  *gl = NULL;
-    off_t offset=0;
+    off_t gts_header_offset=0;
+    off_t offset=0, end_msg_offset=0;
+    char *gts_header=0,*save_gts_header=0;
+    int gtslen=0;
 
     if ( c == NULL ) c = grib_context_get_default();
 
+    gts_header_offset=grib_context_tell( c,f);
     data = wmo_read_bufr_from_file_malloc ( f, 0,&olen,&offset,error );
+    end_msg_offset=grib_context_tell ( c,f );
 
     if ( *error != GRIB_SUCCESS )
     {
@@ -948,6 +964,26 @@ grib_handle* bufr_new_from_file( grib_context* c, FILE* f,int *error )
         return NULL;
     }
 
+    if ( c->gts_header_on )
+    {
+        int g=0;
+        grib_context_seek ( c,gts_header_offset,SEEK_SET,f );
+        gtslen=offset-gts_header_offset;
+        gts_header=(char*)grib_context_malloc ( c,sizeof ( unsigned char ) *gtslen );
+        save_gts_header=gts_header;
+        grib_context_read ( c,gts_header,gtslen,f );
+        g=gtslen;
+        while ( gts_header!=NULL && g != 0 && *gts_header != '\03' )
+        {
+            /*printf("--------%d %X \n",gtslen,*gts_header);*/
+            gts_header++;
+            g--;
+        }
+        if ( g>8 ) {gts_header++;gtslen=g-1;}
+        else gts_header=save_gts_header;
+        grib_context_seek ( c,end_msg_offset,SEEK_SET,f );
+    }
+
     gl = grib_handle_new_from_message ( c, data, olen );
 
     if ( !gl )
@@ -965,6 +1001,17 @@ grib_handle* bufr_new_from_file( grib_context* c, FILE* f,int *error )
     grib_context_increment_handle_total_count(c);
     if (gl->offset == 0) grib_context_set_handle_file_count(c,1);
 
+    if ( c->gts_header_on && gtslen >=8 )
+    {
+        gl->gts_header=(char*)grib_context_malloc ( c,sizeof ( unsigned char ) *gtslen );
+        memcpy ( gl->gts_header,gts_header,gtslen );
+        gl->gts_header_len=gtslen;
+        grib_context_free ( c,save_gts_header );
+        gtslen=0;
+    } else {
+        gl->gts_header=NULL;
+    }
+
     return gl;
 }
 
@@ -1018,8 +1065,10 @@ static grib_handle* grib_handle_new_from_file_no_multi ( grib_context* c,FILE* f
     int gtslen=0;
 
     if ( c == NULL ) c = grib_context_get_default();
+
+    gts_header_offset=grib_context_tell( c,f);
     data = wmo_read_grib_from_file_malloc ( f, headers_only,&olen,&offset,error );
-    end_msg_offset=offset+olen;
+    end_msg_offset=grib_context_tell ( c,f );
 
     if ( *error != GRIB_SUCCESS )
     {
@@ -1076,6 +1125,8 @@ static grib_handle* grib_handle_new_from_file_no_multi ( grib_context* c,FILE* f
         gl->gts_header_len=gtslen;
         grib_context_free ( c,save_gts_header );
         gtslen=0;
+    } else {
+        gl->gts_header=NULL;
     }
 
     return gl;
@@ -1235,6 +1286,47 @@ int grib_get_message_offset ( grib_handle* h,off_t* offset )
     return 0;
 }
 
+int codes_get_product_kind(grib_handle* h, ProductKind* product_kind)
+{
+    if (h) {
+        *product_kind = h->product_kind;
+        return GRIB_SUCCESS;
+    }
+    return GRIB_NULL_HANDLE;
+}
+
+int codes_check_message_header(const void* bytes, size_t length, ProductKind product)
+{
+    const char *p = ((const char*)bytes);
+    Assert(p);
+    Assert(product == PRODUCT_GRIB || product == PRODUCT_BUFR); /* Others not yet implemented */
+    Assert(length > 4);
+    if (product == PRODUCT_GRIB) {
+        if (p[0] != 'G' || p[1] != 'R' || p[2] != 'I' || p[3] != 'B')
+            return GRIB_INVALID_MESSAGE;
+    }
+    else if (product == PRODUCT_BUFR) {
+        if (p[0] != 'B' || p[1] != 'U' || p[2] != 'F' || p[3] != 'R')
+            return GRIB_INVALID_MESSAGE;
+    }
+    else {
+        return GRIB_NOT_IMPLEMENTED;
+    }
+
+    return GRIB_SUCCESS;
+}
+int codes_check_message_footer(const void* bytes, size_t length, ProductKind product)
+{
+    const char *p = ((const char*)bytes);
+    Assert(p);
+    Assert(product == PRODUCT_GRIB || product == PRODUCT_BUFR); /* Others not yet implemented */
+
+    if (p[length-4] != '7' || p[length-3] != '7' || p[length-2] != '7' || p[length-1] != '7') {
+        return GRIB_7777_NOT_FOUND;
+    }
+    return GRIB_SUCCESS;
+}
+
 int grib_get_message_size ( grib_handle* h,size_t* size )
 {
     long totalLength=0;
@@ -1439,18 +1531,6 @@ static void grib2_build_message ( grib_context* context,unsigned char* sections[
     *len=msglen;
 }
 
-void grib_multi_support_on ( grib_context* c )
-{
-    if ( !c ) c=grib_context_get_default();
-    c->multi_support_on=1;
-}
-
-void grib_multi_support_off ( grib_context* c )
-{
-    if ( !c ) c=grib_context_get_default();
-    c->multi_support_on=0;
-}
-
 /* For multi support mode: Reset all file handles equal to f. See GRIB-249 */
 void grib_multi_support_reset_file(grib_context* c, FILE* f)
 {
@@ -1465,36 +1545,6 @@ void grib_multi_support_reset_file(grib_context* c, FILE* f)
     }
 }
 
-void grib_gts_header_on ( grib_context* c )
-{
-    if ( !c ) c=grib_context_get_default();
-    c->gts_header_on=1;
-}
-
-void grib_gts_header_off ( grib_context* c )
-{
-    if ( !c ) c=grib_context_get_default();
-    c->gts_header_on=0;
-}
-
-int grib_get_gribex_mode ( grib_context* c)
-{
-    if ( !c ) c=grib_context_get_default();
-    return c->gribex_mode_on;
-}
-
-void grib_gribex_mode_on ( grib_context* c )
-{
-    if ( !c ) c=grib_context_get_default();
-    c->gribex_mode_on=1;
-}
-
-void grib_gribex_mode_off ( grib_context* c )
-{
-    if ( !c ) c=grib_context_get_default();
-    c->gribex_mode_on=0;
-}
-
 static grib_multi_support* grib_get_multi_support ( grib_context* c, FILE* f )
 {
     int i=0;
@@ -1511,8 +1561,8 @@ static grib_multi_support* grib_get_multi_support ( grib_context* c, FILE* f )
     if ( !gm )
     {
         gm=grib_multi_support_new ( c );
-        if ( !c->multi_support ) c->multi_support=gm;
-        else prev->next=gm;
+        if ( !c->multi_support ) { c->multi_support=gm; }
+        else { if (prev) prev->next=gm; }
     }
 
     gm->next=0;
