@@ -1,5 +1,5 @@
 /*
- * Copyright 2005-2018 ECMWF.
+ * Copyright 2005-2019 ECMWF.
  *
  * This software is licensed under the terms of the Apache Licence Version 2.0
  * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
@@ -695,6 +695,7 @@ static int read_BUFR(reader *r)
     }
 
     if(length==0) {
+        grib_buffer_delete(c,buf);
         return GRIB_INVALID_MESSAGE;
     }
 
@@ -800,10 +801,10 @@ static int read_BUFR(reader *r)
         break;
       default :
         r->seek_from_start(r->read_data,r->offset+4);
+        grib_buffer_delete(c,buf);
         return GRIB_UNSUPPORTED_EDITION;
     }
 
-
     /* Assert(i <= sizeof(tmp)); */
     err=read_the_rest(r, length, tmp, i, 1);
     if (err)
@@ -901,9 +902,9 @@ static int read_any_gts(reader *r)
     int err = 0;
     unsigned char* buffer=NULL;
     unsigned long magic = 0;
-    unsigned long start = 0x010d0d0a;
-    unsigned long theEnd = 0x0d0d0a03;
-    unsigned char tmp[128]={0,}; /* See ECC-735 */
+    unsigned long start = 0x010d0d0a;  /* SOH CR CR LF */
+    unsigned long theEnd = 0x0d0d0a03; /* CR CR LF ETX */
+    unsigned char tmp[1024]={0,}; /* See ECC-735 */
     size_t message_size=0;
     size_t already_read=0;
     int i=0;
@@ -1302,6 +1303,7 @@ int wmo_read_any_from_stream(void* stream_data,long (*stream_proc)(void*,void* b
     return err;
 }
 
+/* This function allocates memory for the result so the user is responsible for freeing it */
 void* wmo_read_any_from_stream_malloc(void* stream_data,long (*stream_proc)(void*,void* buffer,long len) ,size_t *size, int* err)
 {
     alloc_buffer  u;
@@ -1332,7 +1334,7 @@ void* wmo_read_any_from_stream_malloc(void* stream_data,long (*stream_proc)(void
 
 /*================== */
 
-
+/* This function allocates memory for the result so the user is responsible for freeing it */
 void *wmo_read_gts_from_file_malloc(FILE* f,int headers_only,size_t *size,off_t *offset,int* err)
 {
     alloc_buffer u;
@@ -1358,6 +1360,7 @@ void *wmo_read_gts_from_file_malloc(FILE* f,int headers_only,size_t *size,off_t
     return u.buffer;
 }
 
+/* This function allocates memory for the result so the user is responsible for freeing it */
 void *wmo_read_taf_from_file_malloc(FILE* f,int headers_only,size_t *size,off_t *offset,int* err)
 {
     alloc_buffer u;
@@ -1365,6 +1368,8 @@ void *wmo_read_taf_from_file_malloc(FILE* f,int headers_only,size_t *size,off_t
 
     u.buffer       = NULL;
 
+    r.offset          = 0;
+    r.message_size    = 0;
     r.read_data       = f;
     r.read            = &stdio_read;
     r.seek            = &stdio_seek;
@@ -1381,6 +1386,7 @@ void *wmo_read_taf_from_file_malloc(FILE* f,int headers_only,size_t *size,off_t
     return u.buffer;
 }
 
+/* This function allocates memory for the result so the user is responsible for freeing it */
 void *wmo_read_metar_from_file_malloc(FILE* f,int headers_only,size_t *size,off_t *offset,int* err)
 {
     alloc_buffer u;
@@ -1388,7 +1394,9 @@ void *wmo_read_metar_from_file_malloc(FILE* f,int headers_only,size_t *size,off_
 
     u.buffer       = NULL;
 
+    r.message_size    = 0;
     r.read_data       = f;
+    r.offset          = 0;
     r.read            = &stdio_read;
     r.seek            = &stdio_seek;
     r.seek_from_start = &stdio_seek_from_start;
@@ -1404,6 +1412,7 @@ void *wmo_read_metar_from_file_malloc(FILE* f,int headers_only,size_t *size,off_
     return u.buffer;
 }
 
+/* This function allocates memory for the result so the user is responsible for freeing it */
 static void *_wmo_read_any_from_file_malloc(FILE* f,int* err,size_t *size,off_t *offset,
         int grib_ok,int bufr_ok, int hdf5_ok, int wrap_ok, int headers_only)
 {
@@ -1431,22 +1440,23 @@ static void *_wmo_read_any_from_file_malloc(FILE* f,int* err,size_t *size,off_t
 
     return u.buffer;
 }
+
+/* This function allocates memory for the result so the user is responsible for freeing it */
 void *wmo_read_any_from_file_malloc(FILE* f,int headers_only,size_t *size,off_t *offset,int* err)
 {
     return _wmo_read_any_from_file_malloc(f,err,size,offset, 1, 1, 1, 1, headers_only);
 }
-
+/* This function allocates memory for the result so the user is responsible for freeing it */
 void *wmo_read_grib_from_file_malloc(FILE* f,int headers_only,size_t *size,off_t *offset,int* err)
 {
     return _wmo_read_any_from_file_malloc(f,err,size,offset, 1, 0, 0, 0, headers_only);
 }
-
+/* This function allocates memory for the result so the user is responsible for freeing it */
 void *wmo_read_bufr_from_file_malloc(FILE* f,int headers_only,size_t *size,off_t *offset,int* err)
 {
     return _wmo_read_any_from_file_malloc(f,err,size,offset, 0, 1, 0, 0, headers_only);
 }
 
-
 /* ======================================= */
 
 typedef struct context_alloc_buffer {
@@ -1466,7 +1476,6 @@ static void* context_allocate_buffer(void *data,size_t* length,int *err)
     return u->buffer;
 }
 
-
 int grib_read_any_headers_only_from_file(grib_context* ctx,FILE* f,void* buffer,size_t* len)
 {
     int         err;
@@ -1528,7 +1537,6 @@ int grib_read_any_from_file(grib_context* ctx,FILE* f,void* buffer,size_t* len)
 }
 
 /* ======================================= */
-
 typedef struct memory_read_data {
     unsigned char  *data;
     size_t          data_len;
@@ -1664,7 +1672,7 @@ int grib_count_in_filename(grib_context* c, const char* filename, int* n)
     int err=0;
     FILE* fp = NULL;
     if (!c) c=grib_context_get_default();
-    fp = fopen(filename, "r");
+    fp = fopen(filename, "rb");
     if (!fp) {
         grib_context_log(c, GRIB_LOG_ERROR,"grib_count_in_filename: Unable to read file \"%s\"", filename);
         perror(filename);
