@@ -1,5 +1,5 @@
 /*
- * Copyright 2005-2018 ECMWF.
+ * Copyright 2005-2019 ECMWF.
  *
  * This software is licensed under the terms of the Apache Licence Version 2.0
  * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
@@ -167,7 +167,7 @@ static int iterate_reduced_gaussian_subarea_legacy(grib_iterator* iter, grib_han
 
     if (h->context->debug) {
         const size_t np = count_subarea_points(h, get_reduced_row, pl, plsize, lon_first, lon_last);
-        printf("ECCODES DEBUG grib_iterator_class_gaussian_reduced: Legacy sub-area num points=%ld\n", (long)np);
+        fprintf(stderr,"ECCODES DEBUG grib_iterator_class_gaussian_reduced: Legacy sub-area num points=%ld\n", (long)np);
     }
 
     /*find starting latitude */
@@ -219,7 +219,7 @@ static int iterate_reduced_gaussian_subarea_algorithm2(grib_iterator* iter, grib
 
     if (h->context->debug) {
         const size_t np = count_subarea_points(h, get_reduced_row, pl, plsize, lon_first, lon_last);
-        printf("ECCODES DEBUG grib_iterator_class_gaussian_reduced: sub-area num points=%ld\n", (long)np);
+        fprintf(stderr, "ECCODES DEBUG grib_iterator_class_gaussian_reduced: sub-area num points=%ld\n", (long)np);
     }
 
     /* Find starting latitude */
@@ -368,7 +368,7 @@ static int init(grib_iterator* iter, grib_handle* h, grib_arguments* args)
         iter->e=0;
         if (h->context->debug) {
             const size_t np = count_global_points(pl, plsize);
-            printf("ECCODES DEBUG grib_iterator_class_gaussian_reduced: global num points=%ld\n", (long)np);
+            fprintf(stderr, "ECCODES DEBUG grib_iterator_class_gaussian_reduced: global num points=%ld\n", (long)np);
         }
 
         for (j=0;j<plsize;j++) {
