@@ -1,5 +1,5 @@
 /*
- * Copyright 2005-2018 ECMWF.
+ * Copyright 2005-2019 ECMWF.
  *
  * This software is licensed under the terms of the Apache Licence Version 2.0
  * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
@@ -345,22 +345,22 @@ static void print_values(grib_context* c, const grib_util_grid_spec2* spec,
     size_t i=0;
     int isConstant = 1;
     double v = 0, minVal=DBL_MAX, maxVal=-DBL_MAX;
-    printf("ECCODES DEBUG grib_util grib_set_values: setting %lu key/value pairs\n",(unsigned long)count);
+    fprintf(stderr, "ECCODES DEBUG grib_util grib_set_values: setting %lu key/value pairs\n",(unsigned long)count);
 
     for(i=0; i<count; i++)
     {
         switch(keyval_pairs[i].type)
         {
-        case GRIB_TYPE_LONG: printf("ECCODES DEBUG  grib_util: => %s =  %ld;\n"
+        case GRIB_TYPE_LONG: fprintf(stderr, "ECCODES DEBUG  grib_util: => %s =  %ld;\n"
                 ,keyval_pairs[i].name,(long)keyval_pairs[i].long_value); break;
-        case GRIB_TYPE_DOUBLE: printf("ECCODES DEBUG  grib_util: => %s = %.16e;\n"
+        case GRIB_TYPE_DOUBLE: fprintf(stderr, "ECCODES DEBUG  grib_util: => %s = %.16e;\n"
                 ,keyval_pairs[i].name,keyval_pairs[i].double_value); break;
-        case GRIB_TYPE_STRING: printf("ECCODES DEBUG  grib_util: => %s = \"%s\";\n"
+        case GRIB_TYPE_STRING: fprintf(stderr, "ECCODES DEBUG  grib_util: => %s = \"%s\";\n"
                 ,keyval_pairs[i].name,keyval_pairs[i].string_value); break;
         }
     }
 
-    printf("ECCODES DEBUG  grib_util: data_values_count=%lu;\n", (unsigned long)data_values_count);
+    fprintf(stderr, "ECCODES DEBUG  grib_util: data_values_count=%lu;\n", (unsigned long)data_values_count);
     for (i=0; i<data_values_count; i++) {
         if (i==0) v = data_values[i];
         if (data_values[i] != spec->missingValue) {
@@ -380,8 +380,9 @@ static void print_values(grib_context* c, const grib_util_grid_spec2* spec,
             if (v > maxVal) maxVal=v;
         }
     }
-    printf("ECCODES DEBUG  grib_util: data_values are CONSTANT? %d\t(min=%.16e, max=%.16e)\n",
+    fprintf(stderr, "ECCODES DEBUG  grib_util: data_values are CONSTANT? %d\t(min=%.16e, max=%.16e)\n",
            isConstant, minVal, maxVal);
+    if (c->gribex_mode_on) fprintf(stderr, "ECCODES DEBUG  grib_util: GRIBEX mode is turned on!\n");
 
 #if 0
         if (spec->bitmapPresent) {
@@ -522,7 +523,7 @@ static int expand_bounding_box(grib_handle* h, grib_values *values, const size_t
         if (is_angle && !angle_can_be_encoded(h, values[i].double_value)) {
             new_angle = adjust_angle(values[i].double_value, roundingPolicy, angle_subdivisions);
             if (h->context->debug) {
-                printf("ECCODES DEBUG  grib_util EXPAND_BOUNDING_BOX %s: old=%.15e new=%.15e (%s)\n",
+                fprintf(stderr, "ECCODES DEBUG  grib_util EXPAND_BOUNDING_BOX %s: old=%.15e new=%.15e (%s)\n",
                        values[i].name, values[i].double_value, new_angle,
                        (roundingPolicy==eROUND_ANGLE_UP?"Up":"Down"));
             }
@@ -742,6 +743,12 @@ static const char* get_grid_type_name(const int spec_grid_type)
     if (spec_grid_type == GRIB_UTIL_GRID_SPEC_REDUCED_ROTATED_GG)
         return "reduced_rotated_gg";
 
+    if (spec_grid_type == GRIB_UTIL_GRID_SPEC_LAMBERT_AZIMUTHAL_EQUAL_AREA)
+        return "lambert_azimuthal_equal_area";
+
+    if (spec_grid_type == GRIB_UTIL_GRID_SPEC_LAMBERT_CONFORMAL)
+        return "lambert";
+
     return NULL;
 }
 
@@ -850,6 +857,7 @@ grib_handle* grib_util_set_spec2(grib_handle* h,
     int packingTypeIsSet=0;
     int setSecondOrder=0;
     int setJpegPacking=0;
+    int convertEditionEarlier=0;/* For cases when we cannot set some keys without converting */
     size_t slen=17;
     int grib1_high_resolution_fix = 0; /* boolean: See GRIB-863 */
     int global_grid = 0; /* boolean */
@@ -880,9 +888,9 @@ grib_handle* grib_util_set_spec2(grib_handle* h,
     grib_get_long(h,"bitsPerValue",&input_bits_per_value);
     grib_get_long(h,"decimalScaleFactor",&input_decimal_scale_factor);
     if (h->context->debug==-1) {
-        printf("ECCODES DEBUG grib_util: input_packing_type = %s\n",input_packing_type);
-        printf("ECCODES DEBUG grib_util: input_bits_per_value = %ld\n",input_bits_per_value);
-        printf("ECCODES DEBUG grib_util: input_decimal_scale_factor = %ld\n",input_decimal_scale_factor);
+        fprintf(stderr, "ECCODES DEBUG grib_util: input_packing_type = %s\n",input_packing_type);
+        fprintf(stderr, "ECCODES DEBUG grib_util: input_bits_per_value = %ld\n",input_bits_per_value);
+        fprintf(stderr, "ECCODES DEBUG grib_util: input_decimal_scale_factor = %ld\n",input_decimal_scale_factor);
     }
 
     /*if ( (*err=check_values(data_values, data_values_count))!=GRIB_SUCCESS ) {
@@ -923,7 +931,7 @@ grib_handle* grib_util_set_spec2(grib_handle* h,
                 setSecondOrder=1;
                 break;
             default :
-                printf("invalid packing_spec->packing_type = %ld\n",(long)packing_spec->packing_type);
+                fprintf(stderr, "invalid packing_spec->packing_type = %ld\n",(long)packing_spec->packing_type);
                 *err = GRIB_INTERNAL_ERROR;
                 goto cleanup;
                 break;
@@ -949,7 +957,7 @@ grib_handle* grib_util_set_spec2(grib_handle* h,
             break;
 
         default:
-            printf("invalid packing_spec->accuracy = %ld\n",(long)packing_spec->accuracy);
+            fprintf(stderr, "invalid packing_spec->accuracy = %ld\n",(long)packing_spec->accuracy);
             *err = GRIB_INTERNAL_ERROR;
             goto cleanup;
             break;
@@ -976,9 +984,9 @@ grib_handle* grib_util_set_spec2(grib_handle* h,
         }
         if (h->context->debug==-1) {
             int j=0;
-            printf("ECCODES DEBUG grib_util: grib_set_double_array\n");
-            for (j=0;j<20;j++) printf("ECCODES DEBUG grib_util %g\n",data_values[j]);
-            printf("ECCODES DEBUG grib_util: data_values_count=%d \n",(int)data_values_count);
+            fprintf(stderr, "ECCODES DEBUG grib_util: grib_set_double_array\n");
+            for (j=0;j<20;j++) fprintf(stderr, "ECCODES DEBUG grib_util %g\n",data_values[j]);
+            fprintf(stderr, "ECCODES DEBUG grib_util: data_values_count=%d \n",(int)data_values_count);
         }
 
         if((*err = grib_set_double_array(h,"values",data_values,data_values_count)) != 0)
@@ -986,7 +994,7 @@ grib_handle* grib_util_set_spec2(grib_handle* h,
             goto cleanup;
         }
         if (h->context->debug==-1)
-            printf("ECCODES DEBUG grib_util: done grib_set_double_array \n");
+            fprintf(stderr, "ECCODES DEBUG grib_util: done grib_set_double_array \n");
 
 
         /* convert to second_order if not constant field */
@@ -1043,21 +1051,30 @@ grib_handle* grib_util_set_spec2(grib_handle* h,
     /* The "pl" is given from the template, but "section_copy" will take care of setting the right headers */
 
     {
-        char levtype[80];
-        size_t n = sizeof(levtype);
-        Assert(grib_get_string(h,"levelType",levtype,&n) == 0);
         switch (spec->grid_type) {
-        case GRIB_UTIL_GRID_SPEC_REDUCED_GG:
-        case GRIB_UTIL_GRID_SPEC_REDUCED_ROTATED_GG:
-            /* Choose a sample with the right Gaussian number and edition */
-            sprintf(name, "%s_pl_%ld_grib%ld", grid_type,spec->N, editionNumber);
-            if (spec->pl && spec->pl_size) {
-                /* GRIB-834: pl is given so can use any of the reduced_gg_pl samples */
+            case GRIB_UTIL_GRID_SPEC_REDUCED_GG:
+            case GRIB_UTIL_GRID_SPEC_REDUCED_ROTATED_GG:
+                /* Choose a sample with the right Gaussian number and edition */
+                sprintf(name, "%s_pl_%ld_grib%ld", grid_type,spec->N, editionNumber);
+                if (spec->pl && spec->pl_size) {
+                    /* GRIB-834: pl is given so can use any of the reduced_gg_pl samples */
+                    sprintf(name, "%s_pl_grib%ld", grid_type, editionNumber);
+                }
+                break;
+            case GRIB_UTIL_GRID_SPEC_LAMBERT_AZIMUTHAL_EQUAL_AREA:
+                if (editionNumber==1) {   /* This grid type is not available in edition 1 */
+                    if (h->context->debug==-1)
+                        fprintf(stderr,"ECCODES DEBUG grib_util: lambert_azimuthal_equal_area specified "
+                                       "but input is GRIB1. Output must be a higher edition!\n");
+                    convertEditionEarlier=1;
+                }
+                sprintf(name, "GRIB%ld", editionNumber);
+                break;
+            case GRIB_UTIL_GRID_SPEC_LAMBERT_CONFORMAL:
+                sprintf(name, "GRIB%ld", editionNumber);
+                break;
+            default :
                 sprintf(name, "%s_pl_grib%ld", grid_type, editionNumber);
-            }
-            break;
-        default :
-            sprintf(name, "%s_pl_grib%ld", grid_type, editionNumber);
         }
 
         if (spec->pl && spec->grid_name) {
@@ -1173,6 +1190,48 @@ grib_handle* grib_util_set_spec2(grib_handle* h,
 
         break;
 
+    case GRIB_UTIL_GRID_SPEC_LAMBERT_AZIMUTHAL_EQUAL_AREA:
+        COPY_SPEC_LONG  (bitmapPresent);
+        if (spec->missingValue) COPY_SPEC_DOUBLE(missingValue);
+
+        COPY_SPEC_DOUBLE(longitudeOfFirstGridPointInDegrees);
+        COPY_SPEC_DOUBLE(latitudeOfFirstGridPointInDegrees);
+        COPY_SPEC_LONG(Ni); /* same as Nx */
+        COPY_SPEC_LONG(Nj); /* same as Ny */
+        /* TODO
+         * pass in extra keys e.g. Dx, Dy, standardParallel and centralLongitude
+         */
+
+        /*
+        COPY_SPEC_LONG(DxInMetres);
+        COPY_SPEC_LONG(DyInMetres);
+
+        COPY_SPEC_LONG(xDirectionGridLengthInMillimetres);
+        COPY_SPEC_LONG(yDirectionGridLengthInMillimetres);
+        COPY_SPEC_LONG(standardParallelInMicrodegrees);
+        COPY_SPEC_LONG(centralLongitudeInMicrodegrees);
+        */
+
+        break;
+    case GRIB_UTIL_GRID_SPEC_LAMBERT_CONFORMAL:
+        COPY_SPEC_LONG  (bitmapPresent);
+        if (spec->missingValue) COPY_SPEC_DOUBLE(missingValue);
+        COPY_SPEC_DOUBLE(longitudeOfFirstGridPointInDegrees);
+        COPY_SPEC_DOUBLE(latitudeOfFirstGridPointInDegrees);
+        COPY_SPEC_LONG(Ni); /* same as Nx */
+        COPY_SPEC_LONG(Nj); /* same as Ny */
+
+        /*
+         * Note: DxInMetres and DyInMetres
+         * should be 'double' and not integer. WMO GRIB2 uses millimetres!
+         * TODO:
+         * Add other keys like Latin1, LoV etc
+
+         *err = GRIB_NOT_IMPLEMENTED;
+         goto cleanup;
+        */
+        break;
+
     case GRIB_UTIL_GRID_SPEC_REDUCED_GG:
     case GRIB_UTIL_GRID_SPEC_REDUCED_ROTATED_GG:
 
@@ -1274,7 +1333,7 @@ grib_handle* grib_util_set_spec2(grib_handle* h,
             setSecondOrder=1;
             break;
         default :
-            printf("invalid packing_spec->packing_type = %ld\n",(long)packing_spec->packing_type);
+            fprintf(stderr,"invalid packing_spec->packing_type = %ld\n",(long)packing_spec->packing_type);
             *err = GRIB_INTERNAL_ERROR;
             goto cleanup;
             break;
@@ -1331,7 +1390,7 @@ grib_handle* grib_util_set_spec2(grib_handle* h,
         break;
 
     default:
-        printf("invalid packing_spec->accuracy = %ld\n",(long)packing_spec->accuracy);
+        fprintf(stderr, "invalid packing_spec->accuracy = %ld\n",(long)packing_spec->accuracy);
         *err = GRIB_INTERNAL_ERROR;
         goto cleanup;
         break;
@@ -1394,7 +1453,7 @@ grib_handle* grib_util_set_spec2(grib_handle* h,
         if (global_grid) {
             size_t sum = sum_of_pl_array(spec->pl, spec->pl_size);
             if (data_values_count != sum) {
-                printf("invalid reduced gaussian grid: specified as global, data_values_count=%ld but sum of pl array=%ld\n",
+                fprintf(stderr, "invalid reduced gaussian grid: specified as global, data_values_count=%ld but sum of pl array=%ld\n",
                         (long)data_values_count, (long)sum);
                 *err = GRIB_WRONG_GRID;
                 goto cleanup;
@@ -1403,8 +1462,8 @@ grib_handle* grib_util_set_spec2(grib_handle* h,
     }
 
     if (h->context->debug==-1) {
-        printf("ECCODES DEBUG grib_util: global_grid = %d\n", global_grid);
-        printf("ECCODES DEBUG grib_util: expandBoundingBox = %d\n", expandBoundingBox);
+        fprintf(stderr, "ECCODES DEBUG grib_util: global_grid = %d\n", global_grid);
+        fprintf(stderr, "ECCODES DEBUG grib_util: expandBoundingBox = %d\n", expandBoundingBox);
         print_values(h->context,spec,data_values,data_values_count,values,count);
     }
 
@@ -1418,6 +1477,14 @@ grib_handle* grib_util_set_spec2(grib_handle* h,
         }
     }
 
+    if (convertEditionEarlier && packing_spec->editionNumber>1) {
+        *err = grib_set_long(outh,"edition", packing_spec->editionNumber);
+        if(*err) {
+            fprintf(stderr,"SET_GRID_DATA_DESCRIPTION: Cannot convert to edition %ld.\n",packing_spec->editionNumber);
+            goto cleanup;
+        }
+    }
+
     if((*err = grib_set_values(outh,values,count)) != 0)
     {
         fprintf(stderr,"SET_GRID_DATA_DESCRIPTION: Cannot set key values: %s\n",grib_get_error_message(*err));
@@ -1574,7 +1641,7 @@ grib_handle* grib_util_set_spec2(grib_handle* h,
     }
 #endif
     if (h->context->debug==-1)
-        printf("ECCODES DEBUG: grib_util_set_spec end\n");
+        fprintf(stderr, "ECCODES DEBUG: grib_util_set_spec end\n");
 
     return outh;
 
