@@ -1,5 +1,5 @@
 /*
- * Copyright 2005-2018 ECMWF.
+ * Copyright 2005-2019 ECMWF.
  *
  * This software is licensed under the terms of the Apache Licence Version 2.0
  * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
@@ -66,7 +66,7 @@ int grib_set_long_internal(grib_handle* h, const char* name, long val)
     a = grib_find_accessor(h, name);
 
     if (h->context->debug)
-        printf("ECCODES DEBUG grib_set_long_internal %s=%ld\n",name,(long)val);
+        fprintf(stderr, "ECCODES DEBUG grib_set_long_internal %s=%ld\n",name,(long)val);
 
     if(a){
         ret = grib_pack_long(a, &val, &l);
@@ -92,7 +92,7 @@ int grib_set_long(grib_handle* h, const char* name, long val)
     a = grib_find_accessor(h, name);
 
     if (h->context->debug)
-        printf("ECCODES DEBUG grib_set_long %s=%ld\n",name,(long)val);
+        fprintf(stderr, "ECCODES DEBUG grib_set_long %s=%ld\n",name,(long)val);
 
     if(a){
         if(a->flags & GRIB_ACCESSOR_FLAG_READ_ONLY)
@@ -116,7 +116,7 @@ int grib_set_double_internal(grib_handle* h, const char* name, double val)
     a = grib_find_accessor(h, name);
 
     if (h->context->debug)
-        printf("ECCODES DEBUG grib_set_double_internal %s=%g\n",name,val);
+        fprintf(stderr, "ECCODES DEBUG grib_set_double_internal %s=%g\n",name,val);
 
     if(a){
         ret = grib_pack_double(a, &val, &l);
@@ -286,7 +286,8 @@ int grib_copy_namespace(grib_handle* dest, const char* name, grib_handle* src)
             key_err=key_err->next;
         }
     }
-    error_code = *err; /* copy the error code before cleanup */
+    if (err)
+        error_code = *err; /* copy the error code before cleanup */
     grib_keys_iterator_delete(iter);
     key_err=first;
     while (key_err) {
@@ -308,7 +309,7 @@ int grib_set_double(grib_handle* h, const char* name, double val)
     a = grib_find_accessor(h, name);
 
     if (h->context->debug)
-        printf("ECCODES DEBUG grib_set_double %s=%g\n",name,val);
+        fprintf(stderr, "ECCODES DEBUG grib_set_double %s=%g\n",name,val);
 
     if(a){
 
@@ -333,7 +334,7 @@ int grib_set_string_internal(grib_handle* h, const char* name,
     a = grib_find_accessor(h, name);
 
     if (h->context->debug)
-        printf("ECCODES DEBUG grib_set_string_internal %s=%s\n",name,val);
+        fprintf(stderr, "ECCODES DEBUG grib_set_string_internal %s=%s\n",name,val);
 
     if(a){
         ret = grib_pack_string(a, val, length);
@@ -365,7 +366,7 @@ int grib_set_string(grib_handle* h, const char* name, const char* val, size_t *l
         grib_get_long(h,"bitsPerValue",&bitsPerValue);
         if (bitsPerValue==0) {
             if (h->context->debug) {
-                printf("ECCODES DEBUG grib_set_string packingType: Constant field cannot be encoded in second order. Packing not changed\n");
+                fprintf(stderr, "ECCODES DEBUG grib_set_string packingType: Constant field cannot be encoded in second order. Packing not changed\n");
             }
             return 0;
         }
@@ -374,7 +375,7 @@ int grib_set_string(grib_handle* h, const char* name, const char* val, size_t *l
         ret = grib_get_size(h, "codedValues", &numCodedVals);
         if (ret == GRIB_SUCCESS && numCodedVals < 3) {
             if (h->context->debug) {
-                printf("ECCODES DEBUG grib_set_string packingType: not enough coded values for second order. Packing not changed\n");
+                fprintf(stderr, "ECCODES DEBUG grib_set_string packingType: not enough coded values for second order. Packing not changed\n");
             }
             return 0;
         }
@@ -383,7 +384,7 @@ int grib_set_string(grib_handle* h, const char* name, const char* val, size_t *l
     a = grib_find_accessor(h, name);
 
     if (h->context->debug)
-        printf("ECCODES DEBUG grib_set_string %s=|%s|\n",name,val);
+        fprintf(stderr, "ECCODES DEBUG grib_set_string %s=|%s|\n",name,val);
 
     if(a)
     {
@@ -407,7 +408,7 @@ int grib_set_string_array(grib_handle* h, const char* name, const char** val, si
     a = grib_find_accessor(h, name);
 
     if (h->context->debug) {
-        printf("ECCODES DEBUG grib_set_string_array key=%s %ld values\n",name,(long)length);
+        fprintf(stderr, "ECCODES DEBUG grib_set_string_array key=%s %ld values\n",name,(long)length);
     }
 
     if(a)
@@ -520,7 +521,7 @@ int grib_set_missing(grib_handle* h, const char* name)
             return GRIB_READ_ONLY;
 
         if(a->flags & GRIB_ACCESSOR_FLAG_CAN_BE_MISSING) {
-            if (h->context->debug) printf("ECCODES DEBUG grib_set_missing %s\n",name);
+            if (h->context->debug) fprintf(stderr, "ECCODES DEBUG grib_set_missing %s\n",name);
 
             ret=grib_pack_missing(a);
             if(ret == GRIB_SUCCESS)
@@ -625,7 +626,8 @@ static int _grib_set_double_array_internal(grib_handle* h,grib_accessor* a,
                 err  = grib_pack_double(a, val + *encoded_length, &len);
                 *encoded_length += len;
                 if(err == GRIB_SUCCESS){
-                    return grib_dependency_notify_change(a);
+                    /* See ECC-778 */
+                    return _grib_dependency_notify_change(h, a);
                 }
             }
             else {
@@ -661,7 +663,7 @@ static int _grib_set_double_array(grib_handle* h, const char* name,
         err = GRIB_ARRAY_TOO_SMALL;
 
     if(err == GRIB_SUCCESS)
-        return grib_dependency_notify_change(a);
+        return _grib_dependency_notify_change(h,a); /* See ECC-778 */
 
     return err;
 }
@@ -671,7 +673,7 @@ int grib_set_double_array_internal(grib_handle* h, const char* name, const doubl
     int ret=0;
 
     if (h->context->debug)
-        printf("ECCODES DEBUG grib_set_double_array_internal key=%s %ld values\n",name,(long)length);
+        fprintf(stderr, "ECCODES DEBUG grib_set_double_array_internal key=%s %ld values\n",name,(long)length);
 
     if (length==0) {
         grib_accessor* a = grib_find_accessor(h, name);
@@ -683,17 +685,17 @@ int grib_set_double_array_internal(grib_handle* h, const char* name, const doubl
     if (ret!=GRIB_SUCCESS)
         grib_context_log(h->context,GRIB_LOG_ERROR,"unable to set double array %s (%s)",
                 name,grib_get_error_message(ret));
-    /*if (h->context->debug) printf("ECCODES DEBUG grib_set_double_array_internal key=%s --DONE\n",name);*/
+    /*if (h->context->debug) fprintf(stderr,"ECCODES DEBUG grib_set_double_array_internal key=%s --DONE\n",name);*/
     return ret;
 }
 
 static int __grib_set_double_array(grib_handle* h, const char* name, const double* val, size_t length, int check)
 {
-    double v=val[0];
+    double v=0;
     int constant,i;
 
     if (h->context->debug)
-        printf("ECCODES DEBUG grib_set_double_array key=%s %ld values\n",name,(long)length);
+        fprintf(stderr, "ECCODES DEBUG grib_set_double_array key=%s %ld values\n",name,(long)length);
 
     if (length==0) {
         grib_accessor* a = grib_find_accessor(h, name);
@@ -733,15 +735,14 @@ static int __grib_set_double_array(grib_handle* h, const char* name, const doubl
                     !strcmp(packingType,"grid_second_order_SPD2") ||
                     !strcmp(packingType,"grid_second_order_SPD3")
             ) {
-                ret = 0;
                 slen=11; /*length of 'grid_simple' */
                 if (h->context->debug) {
-                    printf("ECCODES DEBUG __grib_set_double_array: Cannot use second order packing for constant fields. Using simple packing\n");
+                    fprintf(stderr, "ECCODES DEBUG __grib_set_double_array: Cannot use second order packing for constant fields. Using simple packing\n");
                 }
                 ret = grib_set_string(h,"packingType","grid_simple",&slen);
                 if (ret != GRIB_SUCCESS) {
                     if (h->context->debug) {
-                        printf("ECCODES DEBUG __grib_set_double_array: could not switch to simple packing!\n");
+                        fprintf(stderr, "ECCODES DEBUG __grib_set_double_array: could not switch to simple packing!\n");
                     }
                 }
             }
@@ -801,7 +802,7 @@ static int _grib_set_long_array(grib_handle* h, const char* name, const long* va
     if (!a) return GRIB_NOT_FOUND ;
 
     if (h->context->debug) {
-        printf("ECCODES DEBUG _grib_set_long_array key=%s %ld values\n",name,(long)length);
+        fprintf(stderr, "ECCODES DEBUG _grib_set_long_array key=%s %ld values\n",name,(long)length);
     }
 
     if (name[0]=='/' || name[0]=='#' ) {
@@ -1157,10 +1158,21 @@ int _grib_get_string_length(grib_accessor* a, size_t* size)
 
 int grib_get_string_length(grib_handle* h, const char* name,size_t* size)
 {
-    grib_accessor* a = grib_find_accessor(h, name);
-    if(!a) return GRIB_NOT_FOUND;
+    grib_accessor* a = NULL;
+    grib_accessors_list* al=NULL;
+    int ret=0;
 
-    return _grib_get_string_length(a,size);
+    if (name[0] == '/' ) {
+        al=grib_find_accessors_list(h,name);
+        if (!al) return GRIB_NOT_FOUND;
+        ret=_grib_get_string_length(al->accessor,size);
+        grib_context_free(h->context,al);
+        return ret;
+    } else {
+        a = grib_find_accessor(h, name);
+        if(!a) return GRIB_NOT_FOUND;
+        return _grib_get_string_length(a,size);
+    }
 }
 
 int _grib_get_size(grib_handle* h, grib_accessor* a,size_t* size)
@@ -1397,7 +1409,9 @@ static int grib_get_key_value(grib_handle* h,grib_key_value_list* kv)
         {
             list->name=grib_keys_iterator_get_name(iter);
             err=grib_get_native_type(h,list->name,&(list->type));
+            if (err) return err;
             err=grib_get_key_value(h,list);
+            if (err) return err;
             list->next=(grib_key_value_list*)grib_context_malloc_clear(h->context,sizeof(grib_key_value_list));
             list=list->next;
         }
@@ -1406,6 +1420,7 @@ static int grib_get_key_value(grib_handle* h,grib_key_value_list* kv)
 
     default:
         err=grib_get_native_type(h,kv->name,&(kv->type));
+        if (err) return err;
         err=grib_get_key_value(h,kv);
         break;
     }
@@ -1565,7 +1580,8 @@ int grib_set_values(grib_handle* h,grib_values* args,size_t count)
         if(args[i].error != GRIB_SUCCESS)
         {
             grib_context_log(h->context,GRIB_LOG_ERROR,
-                    "grib_set_values[%d] %s (%d) failed: %s",  i, args[i].name, args[i].type,
+                    "grib_set_values[%d] %s (type=%s) failed: %s",
+                    i, args[i].name, grib_get_type_name(args[i].type),
                     grib_get_error_message(args[i].error));
             err= err==GRIB_SUCCESS ? args[i].error : err;
         }
@@ -1662,8 +1678,8 @@ int grib_values_check(grib_handle* h, grib_values* values, int count)
 
 int grib_key_equal(grib_handle* h1,grib_handle* h2,const char* key,int type,int *err)
 {
-    double d1,d2;
-    long l1,l2;
+    double d1=0,d2=0;
+    long l1=0,l2=0;
     char s1[500]={0,};
     char s2[500]={0,};
     size_t len1,len2;
@@ -1703,7 +1719,7 @@ int codes_copy_key(grib_handle* h1,grib_handle* h2,const char* key,int type)
     long* al;
     char* s=0;
     char** as=0;
-    size_t len1,len;
+    size_t len1=0,len=0;
     int err=0;
 
     if ( type != GRIB_TYPE_DOUBLE &&
@@ -1752,6 +1768,7 @@ int codes_copy_key(grib_handle* h1,grib_handle* h2,const char* key,int type)
         break;
     case GRIB_TYPE_STRING:
         err=grib_get_string_length(h1,key,&len);
+        if (err) return err;
         if (len1==1) {
             s=(char*)grib_context_malloc_clear(h1->context,len);
             err=grib_get_string(h1,key,s,&len);
