@@ -1,5 +1,5 @@
 /*
- * Copyright 2005-2018 ECMWF.
+ * Copyright 2005-2019 ECMWF.
  *
  * This software is licensed under the terms of the Apache Licence Version 2.0
  * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
@@ -50,7 +50,8 @@ static void test_reduced_gg(int remove_local_def, int edition, const char* packi
     grib_util_grid_spec spec={0,};
     grib_util_packing_spec packing_spec={0,};
 
-    in = fopen(input_filename,"r");     assert(in);
+    assert(input_filename);
+    in = fopen(input_filename,"rb");     assert(in);
     handle = grib_handle_new_from_file(0,in,&err);    assert(handle);
 
     CODES_CHECK(grib_get_string(handle, "gridType", gridType, &slen),0);
@@ -58,7 +59,8 @@ static void test_reduced_gg(int remove_local_def, int edition, const char* packi
         grib_handle_delete(handle);
         return;
     }
-    out = fopen(output_filename,"w");   assert(out);
+    assert(output_filename);
+    out = fopen(output_filename,"wb");   assert(out);
 
     CODES_CHECK(grib_get_size(handle,"values",&inlen), 0);
     values = (double*)malloc(sizeof(double)*inlen);
@@ -110,16 +112,19 @@ static void test_reduced_gg(int remove_local_def, int edition, const char* packi
         h2 = grib_util_set_spec(handle, &spec, &packing_spec, set_spec_flags, values, outlen, &err);
         assert(err == GRIB_INTERNAL_ERROR);
         assert(!h2);
-
+#ifdef INFINITY
         packing_spec.accuracy=GRIB_UTIL_ACCURACY_USE_PROVIDED_BITS_PER_VALUES;
         values[0] = INFINITY;
         h2 = grib_util_set_spec(handle, &spec, &packing_spec, set_spec_flags, values, outlen, &err);
         assert(err == GRIB_ENCODING_ERROR);
         assert(!h2);
+#endif
     }
 
     /* Write out the message to the output file */
     CODES_CHECK(grib_get_message(finalh, &buffer, &size),0);
+    CODES_CHECK(codes_check_message_header(buffer,size,PRODUCT_GRIB),0);
+    CODES_CHECK(codes_check_message_footer(buffer,size,PRODUCT_GRIB),0);
     if(fwrite(buffer,1,size,out) != size) {
         assert(0);
     }
@@ -151,7 +156,8 @@ static void test_regular_ll(int remove_local_def, int edition, const char* packi
     grib_util_grid_spec spec={0,};
     grib_util_packing_spec packing_spec={0,};
 
-    in = fopen(input_filename,"r");     assert(in);
+    assert(input_filename);
+    in = fopen(input_filename,"rb");     assert(in);
     handle = codes_handle_new_from_file(0, in, PRODUCT_GRIB, &err);    assert(handle);
     
     CODES_CHECK(codes_get_long(handle, "edition", &input_edition), 0);
@@ -161,7 +167,8 @@ static void test_regular_ll(int remove_local_def, int edition, const char* packi
         grib_handle_delete(handle);
         return;
     }
-    out = fopen(output_filename,"w");   assert(out);
+    assert(output_filename);
+    out = fopen(output_filename,"wb");   assert(out);
 
     CODES_CHECK(codes_get_size(handle,"values",&inlen), 0);
     values = (double*)malloc(sizeof(double)*inlen);
@@ -220,6 +227,8 @@ static void test_regular_ll(int remove_local_def, int edition, const char* packi
 
     /* Write out the message to the output file */
     CODES_CHECK(codes_get_message(finalh, &buffer, &size),0);
+    CODES_CHECK(codes_check_message_header(buffer,size,PRODUCT_GRIB),0);
+    CODES_CHECK(codes_check_message_footer(buffer,size,PRODUCT_GRIB),0);
     if(fwrite(buffer,1,size,out) != size) {
         assert(0);
     }
@@ -250,7 +259,7 @@ static void test_grid_complex_spatial_differencing(int remove_local_def, int edi
     grib_util_grid_spec spec={0,};
     grib_util_packing_spec packing_spec={0,};
 
-    in = fopen(input_filename,"r");     assert(in);
+    in = fopen(input_filename,"rb");     assert(in);
     handle = codes_handle_new_from_file(0, in, PRODUCT_GRIB, &err);    assert(handle);
 
     CODES_CHECK(grib_get_string(handle, "packingType", gridType, &slen),0);
@@ -258,7 +267,7 @@ static void test_grid_complex_spatial_differencing(int remove_local_def, int edi
         grib_handle_delete(handle);
         return;
     }
-    out = fopen(output_filename,"w");   assert(out);
+    out = fopen(output_filename,"wb");   assert(out);
 
     CODES_CHECK(codes_get_size(handle,"values",&inlen), 0);
     values = (double*)malloc(sizeof(double)*inlen);
@@ -329,40 +338,38 @@ static void usage(const char *prog)
 
 int main(int argc, char *argv[])
 {
-    int opt = 0, remove_local_def = 0;
+    int i = 0, remove_local_def = 0;
     int edition = 0;
     char* packingType = NULL;
     const char* prog = argv[0];
     char* infile_name = NULL;
     char* outfile_name = NULL;
     
-    while ((opt = getopt(argc, argv, "re:p:")) != -1) {
-        switch (opt) {
-            case 'r':
-                remove_local_def=1;
-                break;
-            case 'p':
-                packingType = optarg;
-                break;
-            case 'e':
-                edition = atoi(optarg);
-                break;
-            default:
-                usage(prog);
-                break;
+    if (argc==1 || argc >8) usage(prog);
+
+    for (i = 1; i < argc; i++) {
+        if (strcmp(argv[i],"-p")==0) {
+            packingType = argv[i+1];
+            ++i;
+        } else if (strcmp(argv[i],"-e")==0) {
+            edition = atoi( argv[i+1] );
+            ++i;
+        } else if (strcmp(argv[i],"-r")==0) {
+            remove_local_def = 1;
+        } else {
+            /* Expect 2 filenames */
+            infile_name = argv[i];
+            outfile_name = argv[i+1];
+            break;
         }
     }
-
-    /* After option processing expect just two files */
-    if (argc-optind != 2) usage(prog);
-
-    /*for (i = optind; i < argc; i++) {
-        printf ("File argument %s\n", argv[i]);
-    }*/
-
-    infile_name = argv[argc-2];
-    outfile_name = argv[argc-1];
-
+#if 0
+    printf("DEBUG remove_local_def = %d\n", remove_local_def);
+    printf("DEBUG edition          = %d\n", edition);
+    printf("DEBUG packingType      = %s\n", packingType);
+    printf("DEBUG infile_name      = %s\n", infile_name);
+    printf("DEBUG outfile_name     = %s\n", outfile_name);
+#endif
     test_regular_ll(remove_local_def, edition, packingType, infile_name, outfile_name);
     test_reduced_gg(remove_local_def, edition, packingType, infile_name, outfile_name);
     /*test_grid_complex_spatial_differencing(remove_local_def, edition, packingType, infile_name, outfile_name);*/
