@@ -1,5 +1,5 @@
 /*
- * Copyright 2005-2018 ECMWF.
+ * Copyright 2005-2019 ECMWF.
  *
  * This software is licensed under the terms of the Apache Licence Version 2.0
  * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
@@ -14,16 +14,18 @@
  */
 #include "grib_tools.h"
 
+extern char *optarg;
+extern int optind;
+
 #ifdef ECCODES_ON_WINDOWS
-/* Microsoft Windows Visual Studio support */
 #include "wingetopt.h"
 #endif
 
-const char* names[]={"parameter","vertical","geography","data","mars","local"};
-int names_count=6;
+static const char* names[]={"parameter","vertical","geography","data","mars","local"};
+static int names_count=6;
 
 /* id,args,help */
-grib_options_help grib_options_help_list[] ={
+static grib_options_help grib_options_help_list[] ={
   {"a",0,"Dump aliases.\n"},
   {"b:","key,key,...",
      "\n\t\tAll the keys in this list are skipped in the comparison. Bit-by-bit compare on.\n"},
@@ -45,10 +47,10 @@ grib_options_help grib_options_help_list[] ={
     "\n\t\tData value corresponding to the given index is printed.\n"},
   {"j",0,"JSON mode (JavaScript Object Notation).\n"},
   {"l:","Latitude,Longitude[,MODE,file]",
-   "\n\t\tValue close to the point of a Latitude/Longitude."
+   "\n\t\tValue close to the point of a Latitude,Longitude."
    "\n\t\tAllowed values for MODE are:"
-   "\n\t\t4 (4 values in the nearest points are printed) Default"
-   "\n\t\t1 (the value at the nearest point is printed)"
+   "\n\t\t  4 (4 values in the nearest points are printed) Default"
+   "\n\t\t  1 (the value at the nearest point is printed)"
    "\n\t\tfile (file is used as mask. The closer point with mask value>=0.5 is printed)\n"
        },
   {"n:","namespace",
@@ -98,7 +100,7 @@ grib_options_help grib_options_help_list[] ={
         "\n\t\tall=relative_error will compare all the floating-point keys using relative_error. Default all=0.\n"},
   {"S",0,"Strict. Only messages matching all the constraints are copied to"
    "\n\t\tthe output file\n"},
-  {"T:","T | B | M | A","Message type. T->GTS, B->BUFR, M->METAR (Experimental),A->Any (Experimental).\n\t\t\tThe input file is interpreted according to the message type.\n"},
+  {"T:","T | B | M | A","Message type. T->GTS, B->BUFR, M->METAR (Experimental), A->Any (Experimental).\n\t\t\t\tThe input file is interpreted according to the message type.\n"},
   {"V",0,"Version.\n"},
   {"W:","width","\n\t\tMinimum width of each column in output. Default is 10.\n"},
   {"X:","offset","\n\t\tInput file offset in bytes. Processing of the input file will start from \"offset\".\n"},
@@ -109,7 +111,7 @@ grib_options_help grib_options_help_list[] ={
 
 };
 
-int grib_options_help_count=sizeof(grib_options_help_list)/sizeof(grib_options_help);
+static int grib_options_help_count=sizeof(grib_options_help_list)/sizeof(grib_options_help);
 
 
 void usage(void)
@@ -204,9 +206,10 @@ int grib_process_runtime_options(grib_context* context,int argc,char** argv,grib
 
     if (grib_options_on("T:")) {
         char* x=grib_options_get_option("T:");
-        if ( *x == 'T' ) options->mode=MODE_GTS;
+        if      ( *x == 'T' ) options->mode=MODE_GTS;
         else if ( *x == 'B' ) options->mode=MODE_BUFR;
         else if ( *x == 'M' ) options->mode=MODE_METAR;
+        else if ( *x == 'F' ) options->mode=MODE_TAF;
         else if ( *x == 'A' ) options->mode=MODE_ANY;
         else options->mode=MODE_GRIB;
     }
@@ -224,6 +227,9 @@ int grib_process_runtime_options(grib_context* context,int argc,char** argv,grib
     if (grib_options_on("l:"))
         options->latlon=grib_options_get_option("l:");
 
+    if (grib_options_on("j")) options->json_output=1;
+    else options->json_output=0;
+
     if (grib_options_on("X:"))
         options->infile_offset=atol(grib_options_get_option("X:"));
 
@@ -238,7 +244,13 @@ int grib_process_runtime_options(grib_context* context,int argc,char** argv,grib
     options->gts=grib_options_on("g");
 
     if (grib_options_on("d:")) {
-        options->constant=atof(grib_options_get_option("d:"));
+        char* endPtr = NULL; /* for error handling */
+        const char* optionStr = grib_options_get_option("d:");
+        options->constant=strtod(optionStr, &endPtr);
+        if(*endPtr) {
+            fprintf(stderr, "Invalid number for -d option: '%s'\n", optionStr);
+            exit(1);
+        }
         options->repack=1;
     }
 
